#!/usr/bin/env python
'''
Created on 16/04/2018

miRmachine walk on tree, parse out node miRNAs


@author: suu13
'''

from __future__ import print_function
import re
from docopt import docopt
import newick


__author__ = 'sium'

__licence__="""
MIT License

Copyright (c) 2018 Sinan Ugur Umu (SUU) sinanugur@gmail.com

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""

__doc__="""Parse tree to find node miRNAs.

Usage:
    mirmachine-tree-parser.py <newick> <keyword> [--add-all-nodes]
    mirmachine-tree-parser.py <newick> --print-all-nodes
    mirmachine-tree-parser.py (-h | --help)
    mirmachine-tree-parser.py --version

Arguments:
    newick                              A newick tree.
    keyword                             A keyword to gather node miRNAs (e.g. Homo).

Options:
    -a --add-all-nodes                 Move on the tree both ways.
    -p --print-all-nodes               Print all available nodes and exit.
    -h --help                          Show this screen.
    --version                          Show version.

"""


def detect_ancestors(node,ancestors):
    if node == None:
        return
    else:
        if node.name is not None:
            ancestors.extend([x.strip() for x in node.name.split("_")])
        detect_ancestors(node.ancestor,ancestors)

    return

def detect_descendants(node,descendants):
    if not node:
        return
    else:
        for i in node:
            if i.name is not None and not i.is_leaf:
                descendants.extend([x.strip() for x in i.name.split("_")])
            detect_descendants(i.descendants,descendants)

    return

def walk_on_tree(newick_file):
    descendants=[]
    tree=newick.read(newick_file)
    for node in tree[0].walk():
        if node.name is not None and not node.is_leaf:
            y=[x.strip() for x in node.name.split("_")]
            descendants.extend(list(filter(lambda x: len(x) > 2,y)))
    
    while "group" in descendants: descendants.remove("group") 
    for d in descendants:
        print(d)

    return



def search_tree_for_keyword(newick_file,keyword):

    ancestors=[]
    descendants=[]
    tree=newick.read(newick_file)
    for node in tree[0].walk():
        if node.name is not None and node.name.find(keyword.strip()) != -1:
            detect_ancestors(node, ancestors)
            detect_descendants([node], descendants)
            while "group" in descendants: descendants.remove("group") 
            while "group" in ancestors: ancestors.remove("group") 
            
            if arguments['--add-all-nodes']:
                for d in descendants:
                    print(d)
            for a in ancestors:
                print(a)

            break

    return






def main():
    if arguments["--print-all-nodes"]:
        walk_on_tree(arguments['<newick>'])
    else:
        search_tree_for_keyword(arguments['<newick>'],arguments['<keyword>'])


if __name__ == '__main__':
    arguments = docopt(__doc__, version='0.97')
    main()



#!/usr/bin/env python
'''
Created on 03/08/2020

MirMachine main

@author: Sinan U. Umu, sinanugur@gmail.com
'''

#from __future__ import print_function
import re
from docopt import docopt
import newick
import os
import subprocess
from pathlib import Path
from datetime import datetime

try:
    from mirmachine import meta
    from mirmachine import workflows
    import mirmachine
    mirmachine_path=os.path.dirname(mirmachine.__file__)
except ImportError:
    try:
        import meta
        import workflows
        mirmachine_path="mirmachine" #so you did not install the package
    except:
            raise ImportError


meta_directory=os.path.dirname(meta.__file__)

__author__ = 'sium'

__licence__="""
MIT License

Copyright (c) 2020 Sinan Ugur Umu (SUU) sinanugur@gmail.com

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""

__doc__="""Main MirMachine executable

Usage:
    MirMachine.py --node <text> --species <text> --genome <text> [--model <text>] [--cpu <integer>] [--add-all-nodes|--single-node-only] [--dry]
    MirMachine.py --species <text> --genome <text> --family <text> [--model <text>] [--dry]
    MirMachine.py --print-all-nodes
    MirMachine.py (-h | --help)
    MirMachine.py --version

Arguments:
    -n <text>, --node <text>              Node name. (e.g. Caenorhabditis)
    -s <text>, --species <text>           Species name. (e.g. Caenorhabditis_elegans)
    -g <text>, --genome <text>            Genome fasta file location (e.g. data/genome/example.fasta)
    -m <text>, --model <text>             Model type: deutero, proto, combined [default: proto]
    -f <text>, --family <text>            Run only a single miRNA family.
    -c <integer>, --cpu <integer>         CPUs. [default: 2]

Options:
    -a --add-all-nodes                 Move on the tree both ways.
    -o --single-node-only              Run only the given node.
    -d --dry                           Dry run.
    -p --print-all-nodes               Print all available nodes and exit.
    -h --help                          Show this screen.
    --version                          Show version.

"""




def run_mirmachine():

    Path("data/yamls").mkdir(parents=True,exist_ok=True)

    both_ways= "--add-all-nodes" if arguments["--add-all-nodes"] else ""
    dry_run="-n" if arguments["--dry"] else ""
    default_node_argument= "" if arguments["--single-node-only"] else "| while read i; do mirmachine-tree-parser.py {meta_directory}/tree.newick $i {both_ways}; done".format(meta_directory=meta_directory,both_ways=both_ways)

    if arguments['--family']:
        yaml_argument = """echo {family} | awk -v genome={genome} -v species={species} 'BEGIN{{print "genome: "genome;print "species: "species;print "node: "node; print "mirnas:"}}{{print " - "$1}}' > data/yamls/{species}.yaml""".format(
        family=arguments['--family'],
          species=arguments['--species'],
          genome=arguments['--genome'])
    else:
        yaml_argument="""echo {node} {default_node_argument} | sort | uniq | while read a; \
        do grep $a {meta_directory}/nodes_mirnas_corrected.tsv; done \
        | grep -v NOVEL | grep -v NA | cut -f2 | sort | uniq | \
        awk -v genome={genome} -v species={species} -v node={node} 'BEGIN{{print "genome: "genome;print "species: "species;print "node: "node; print "mirnas:"}}{{print " - "$1}}' > data/yamls/{species}.yaml""".format(
          default_node_argument=default_node_argument,
          meta_directory=meta_directory,
          node=arguments['--node'],
            mirmachine_path=mirmachine_path,
          species=arguments['--species'],
          genome=arguments['--genome'])

    
    
    subprocess.check_call(yaml_argument,shell=True)

    snakemake_argument="snakemake {dry} -j {cpu} -s {mirmachine_path}/workflows/mirmachine_search.smk --config meta_directory={meta_directory} model={model} mirmachine_path={mirmachine_path} --configfile=data/yamls/{species}.yaml".format(
    species=arguments['--species'],
    cpu=arguments['--cpu'],
    model=arguments['--model'],
    meta_directory=meta_directory,
    mirmachine_path=mirmachine_path,
    dry=dry_run)
    
    subprocess.call(snakemake_argument,shell=True)


def print_all_nodes():
        tree_parser_argument="mirmachine-tree-parser.py {meta_directory}/tree.newick --print-all-nodes".format(meta_directory=meta_directory,mirmachine_path=mirmachine_path)
        subprocess.call(tree_parser_argument,shell=True)

def main():
    if arguments["--print-all-nodes"]:
        print_all_nodes()
    else:
        start_time = datetime.now()
        run_mirmachine()
        end_time = datetime.now()
        print('Total runtime: {}'.format(end_time - start_time))


if __name__ == '__main__':
    arguments = docopt(__doc__, version='0.2.1a')
    main()



from setuptools import setup, find_packages, Command
import os

with open("requirements.txt") as req:
    requirements=req.readlines()

class CleanCommand(Command):
    """Custom clean command to tidy up the project root."""
    user_options = []
    def initialize_options(self):
        pass
    def finalize_options(self):
        pass
    def run(self):
        os.system('rm -vrf ./build ./dist ./*.pyc ./*.tgz ./*.egg-info')


with open("README.md","r") as readme:
    long_description=readme.read()

setup(
    name="MirMachine",
    version="0.2.1a",
    packages=find_packages(exclude=('tests*','testing*')),
    scripts=["scripts/MirMachine.py","scripts/mirmachine-tree-parser.py","scripts/gff_sort_and_compete.sh"],
    long_description=long_description,
    long_description_content_type="text/markdown",
    extras_require={"dev":["pytest>=3.7"]},
    install_requires=requirements,
    include_package_data=True,
    zip_safe=False,
    #data_files={"meta":["*.tsv"]},
    #package_data={'meta': ["*.tsv",'meta/*.tsv'], "workflows" : ["*.smk","workflows/*.smk"]},
    #data_files=[("mirmachine",["meta/cms/proto/*.CM"])],
    # metadata to display on PyPI
    author="Sinan U. Umu",
    author_email="sinanugur@gmail.com",
    description="MirMachine",
    keywords="RNA miRNA detection prediction",
        cmdclass={
        'clean': CleanCommand,
    },
    url="https://github.com/sinanugur/MirMachine",   # project home page, if any
    classifiers=[
        'License :: OSI Approved :: MIT License',
        "Programming Language :: Python :: 3",
        'Topic :: Scientific/Engineering :: Bio-Informatics'

    ]

    # could also include long_description, download_url, etc.
)
